# 1 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2025 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \note
!>      If parallel mode is distributed certain combination of
!>      "in_use" and "in_space" can not be used.
!>      For performance reasons it would be better to have the loops
!>      over g-vectros in the gather/scatter routines in new subprograms
!>      with the actual arrays (also the addressing) in the parameter list
!> \par History
!>      JGH (29-Dec-2000) : Changes for parallel use
!>      JGH (13-Mar-2001) : added timing calls
!>      JGH (26-Feb-2003) : OpenMP enabled
!>      JGH (17-Nov-2007) : Removed mass arrays
!>      JGH (01-Dec-2007) : Removed and renamed routines
!>      03.2008 [tlaino] : Splitting pw_types into pw_types and pw_methods
!> \author apsi
! **************************************************************************************************
MODULE pw_types

   USE cp_log_handling, ONLY: cp_to_string
   USE kinds, ONLY: dp
   USE pw_grid_types, ONLY: pw_grid_type
   USE pw_grids, ONLY: pw_grid_release, &
                       pw_grid_retain
#include "../base/base_uses.f90"

   IMPLICIT NONE

# 1 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp"
# 19 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp"

# 32 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp"

# 34 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp"
# 35 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.fypp"
# 36 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F" 2

   PRIVATE
# 39 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_r1d_rs_type, pw_r1d_rs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_r3d_rs_type, pw_r3d_rs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_c1d_rs_type, pw_c1d_rs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_c3d_rs_type, pw_c3d_rs_p_type
# 42 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 39 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_r1d_gs_type, pw_r1d_gs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_r3d_gs_type, pw_r3d_gs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_c1d_gs_type, pw_c1d_gs_p_type
# 40 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
         PUBLIC :: pw_c3d_gs_type, pw_c3d_gs_p_type
# 42 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 43 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"

# 45 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_r1d_rs_type
            REAL(KIND=dp), DIMENSION(:), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_r1d_rs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_r1d_rs
         END TYPE pw_r1d_rs_type

! **************************************************************************************************
         TYPE pw_r1d_rs_p_type
            TYPE(pw_r1d_rs_type), POINTER :: pw => NULL()
         END TYPE pw_r1d_rs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_r3d_rs_type
            REAL(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_r3d_rs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_r3d_rs
         END TYPE pw_r3d_rs_type

! **************************************************************************************************
         TYPE pw_r3d_rs_p_type
            TYPE(pw_r3d_rs_type), POINTER :: pw => NULL()
         END TYPE pw_r3d_rs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_c1d_rs_type
            COMPLEX(KIND=dp), DIMENSION(:), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_c1d_rs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_c1d_rs
         END TYPE pw_c1d_rs_type

! **************************************************************************************************
         TYPE pw_c1d_rs_p_type
            TYPE(pw_c1d_rs_type), POINTER :: pw => NULL()
         END TYPE pw_c1d_rs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_c3d_rs_type
            COMPLEX(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_c3d_rs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_c3d_rs
         END TYPE pw_c3d_rs_type

! **************************************************************************************************
         TYPE pw_c3d_rs_p_type
            TYPE(pw_c3d_rs_type), POINTER :: pw => NULL()
         END TYPE pw_c3d_rs_p_type
# 60 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 45 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_r1d_gs_type
            REAL(KIND=dp), DIMENSION(:), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_r1d_gs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_r1d_gs
         END TYPE pw_r1d_gs_type

! **************************************************************************************************
         TYPE pw_r1d_gs_p_type
            TYPE(pw_r1d_gs_type), POINTER :: pw => NULL()
         END TYPE pw_r1d_gs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_r3d_gs_type
            REAL(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_r3d_gs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_r3d_gs
         END TYPE pw_r3d_gs_type

! **************************************************************************************************
         TYPE pw_r3d_gs_p_type
            TYPE(pw_r3d_gs_type), POINTER :: pw => NULL()
         END TYPE pw_r3d_gs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_c1d_gs_type
            COMPLEX(KIND=dp), DIMENSION(:), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_c1d_gs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_c1d_gs
         END TYPE pw_c1d_gs_type

! **************************************************************************************************
         TYPE pw_c1d_gs_p_type
            TYPE(pw_c1d_gs_type), POINTER :: pw => NULL()
         END TYPE pw_c1d_gs_p_type
# 46 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
         TYPE pw_c3d_gs_type
            COMPLEX(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, POINTER :: array => NULL()
            TYPE(pw_grid_type), POINTER :: pw_grid => NULL()
         CONTAINS
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: create => pw_create_c3d_gs
            PROCEDURE, PUBLIC, NON_OVERRIDABLE :: release => pw_release_c3d_gs
         END TYPE pw_c3d_gs_type

! **************************************************************************************************
         TYPE pw_c3d_gs_p_type
            TYPE(pw_c3d_gs_type), POINTER :: pw => NULL()
         END TYPE pw_c3d_gs_p_type
# 60 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 61 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"

   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'pw_types'
   LOGICAL, PARAMETER, PRIVATE :: debug_this_module = .FALSE.

CONTAINS
# 67 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_r1d_rs (pw)
            CLASS(pw_r1d_rs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_r1d_rs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_r1d_rs (pw, pw_grid, array_ptr)
            CLASS(pw_r1d_rs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            REAL(KIND=dp), DIMENSION(:), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_r1d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 111 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            IF (PRESENT(array_ptr)) THEN
            IF (ASSOCIATED(array_ptr)) THEN
               CPASSERT(SIZE(array_ptr) == pw%pw_grid%ngpts_cut_local)
               pw%array(1:pw%pw_grid%ngpts_cut_local) => array_ptr
            END IF
            END IF
            IF (.NOT. ASSOCIATED(pw%array)) ALLOCATE (pw%array(pw%pw_grid%ngpts_cut_local))
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_r1d_rs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_r3d_rs (pw)
            CLASS(pw_r3d_rs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_r3d_rs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_r3d_rs (pw, pw_grid, array_ptr)
            CLASS(pw_r3d_rs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            REAL(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_r3d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 119 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            ASSOCIATE (bounds => pw%pw_grid%bounds_local)
               IF (PRESENT(array_ptr)) THEN
                  IF (ASSOCIATED(array_ptr)) THEN
                     IF (ALL(bounds(1, :) <= bounds(2, :))) THEN
                        CPASSERT(ALL(LBOUND(array_ptr) == bounds(1, :)))
                        CPASSERT(ALL(UBOUND(array_ptr) == bounds(2, :)))
                     END IF
                     pw%array => array_ptr
                  END IF
               END IF
               IF (.NOT. ASSOCIATED(pw%array)) THEN
                  ALLOCATE (pw%array( &
                            bounds(1, 1):bounds(2, 1), &
                            bounds(1, 2):bounds(2, 2), &
                            bounds(1, 3):bounds(2, 3)))
               END IF
            END ASSOCIATE
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_r3d_rs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_c1d_rs (pw)
            CLASS(pw_c1d_rs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_c1d_rs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_c1d_rs (pw, pw_grid, array_ptr)
            CLASS(pw_c1d_rs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            COMPLEX(KIND=dp), DIMENSION(:), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_c1d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 111 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            IF (PRESENT(array_ptr)) THEN
            IF (ASSOCIATED(array_ptr)) THEN
               CPASSERT(SIZE(array_ptr) == pw%pw_grid%ngpts_cut_local)
               pw%array(1:pw%pw_grid%ngpts_cut_local) => array_ptr
            END IF
            END IF
            IF (.NOT. ASSOCIATED(pw%array)) ALLOCATE (pw%array(pw%pw_grid%ngpts_cut_local))
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_c1d_rs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_c3d_rs (pw)
            CLASS(pw_c3d_rs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_c3d_rs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_c3d_rs (pw, pw_grid, array_ptr)
            CLASS(pw_c3d_rs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            COMPLEX(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_c3d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 119 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            ASSOCIATE (bounds => pw%pw_grid%bounds_local)
               IF (PRESENT(array_ptr)) THEN
                  IF (ASSOCIATED(array_ptr)) THEN
                     IF (ALL(bounds(1, :) <= bounds(2, :))) THEN
                        CPASSERT(ALL(LBOUND(array_ptr) == bounds(1, :)))
                        CPASSERT(ALL(UBOUND(array_ptr) == bounds(2, :)))
                     END IF
                     pw%array => array_ptr
                  END IF
               END IF
               IF (.NOT. ASSOCIATED(pw%array)) THEN
                  ALLOCATE (pw%array( &
                            bounds(1, 1):bounds(2, 1), &
                            bounds(1, 2):bounds(2, 2), &
                            bounds(1, 3):bounds(2, 3)))
               END IF
            END ASSOCIATE
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_c3d_rs
# 140 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 67 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_r1d_gs (pw)
            CLASS(pw_r1d_gs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_r1d_gs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_r1d_gs (pw, pw_grid, array_ptr)
            CLASS(pw_r1d_gs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            REAL(KIND=dp), DIMENSION(:), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_r1d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 111 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            IF (PRESENT(array_ptr)) THEN
            IF (ASSOCIATED(array_ptr)) THEN
               CPASSERT(SIZE(array_ptr) == pw%pw_grid%ngpts_cut_local)
               pw%array(1:pw%pw_grid%ngpts_cut_local) => array_ptr
            END IF
            END IF
            IF (.NOT. ASSOCIATED(pw%array)) ALLOCATE (pw%array(pw%pw_grid%ngpts_cut_local))
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_r1d_gs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_r3d_gs (pw)
            CLASS(pw_r3d_gs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_r3d_gs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_r3d_gs (pw, pw_grid, array_ptr)
            CLASS(pw_r3d_gs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            REAL(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_r3d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 119 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            ASSOCIATE (bounds => pw%pw_grid%bounds_local)
               IF (PRESENT(array_ptr)) THEN
                  IF (ASSOCIATED(array_ptr)) THEN
                     IF (ALL(bounds(1, :) <= bounds(2, :))) THEN
                        CPASSERT(ALL(LBOUND(array_ptr) == bounds(1, :)))
                        CPASSERT(ALL(UBOUND(array_ptr) == bounds(2, :)))
                     END IF
                     pw%array => array_ptr
                  END IF
               END IF
               IF (.NOT. ASSOCIATED(pw%array)) THEN
                  ALLOCATE (pw%array( &
                            bounds(1, 1):bounds(2, 1), &
                            bounds(1, 2):bounds(2, 2), &
                            bounds(1, 3):bounds(2, 3)))
               END IF
            END ASSOCIATE
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_r3d_gs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_c1d_gs (pw)
            CLASS(pw_c1d_gs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_c1d_gs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_c1d_gs (pw, pw_grid, array_ptr)
            CLASS(pw_c1d_gs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            COMPLEX(KIND=dp), DIMENSION(:), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_c1d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 111 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            IF (PRESENT(array_ptr)) THEN
            IF (ASSOCIATED(array_ptr)) THEN
               CPASSERT(SIZE(array_ptr) == pw%pw_grid%ngpts_cut_local)
               pw%array(1:pw%pw_grid%ngpts_cut_local) => array_ptr
            END IF
            END IF
            IF (.NOT. ASSOCIATED(pw%array)) ALLOCATE (pw%array(pw%pw_grid%ngpts_cut_local))
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_c1d_gs
# 68 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
! **************************************************************************************************
!> \brief releases the given pw
!> \param pw the pw to release
!> \par History
!>      04.2003 created [fawzi]
!> \author fawzi
!> \note
!>      see doc/ReferenceCounting.html
! **************************************************************************************************
         SUBROUTINE pw_release_c3d_gs (pw)
            CLASS(pw_c3d_gs_type), INTENT(INOUT)                       :: pw

            IF (ASSOCIATED(pw%array)) DEALLOCATE (pw%array)
            CALL pw_grid_release(pw%pw_grid)
         END SUBROUTINE pw_release_c3d_gs

! **************************************************************************************************
!> \brief allocates and initializes pw_r3d_rs_type
!> \param pw the type that will bw allocated and initialized
!> \param pw_grid ...
!> \param array_ptr pointer with to data
!> \par History
!>      11.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
         SUBROUTINE pw_create_c3d_gs (pw, pw_grid, array_ptr)
            CLASS(pw_c3d_gs_type), INTENT(INOUT)                         :: pw
            TYPE(pw_grid_type), INTENT(IN), POINTER            :: pw_grid
            COMPLEX(KIND=dp), DIMENSION(:, :, :), CONTIGUOUS, INTENT(IN), OPTIONAL, POINTER                   :: array_ptr

            CHARACTER(len=*), PARAMETER                        :: routineN = 'pw_create_c3d'

            INTEGER                                            :: handle

            CALL timeset(routineN, handle)

            ! Ensure a clean grid to prevent memory leaks
            CALL pw%release()

            pw%pw_grid => pw_grid
            CALL pw_grid_retain(pw%pw_grid)

# 119 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            ASSOCIATE (bounds => pw%pw_grid%bounds_local)
               IF (PRESENT(array_ptr)) THEN
                  IF (ASSOCIATED(array_ptr)) THEN
                     IF (ALL(bounds(1, :) <= bounds(2, :))) THEN
                        CPASSERT(ALL(LBOUND(array_ptr) == bounds(1, :)))
                        CPASSERT(ALL(UBOUND(array_ptr) == bounds(2, :)))
                     END IF
                     pw%array => array_ptr
                  END IF
               END IF
               IF (.NOT. ASSOCIATED(pw%array)) THEN
                  ALLOCATE (pw%array( &
                            bounds(1, 1):bounds(2, 1), &
                            bounds(1, 2):bounds(2, 2), &
                            bounds(1, 3):bounds(2, 3)))
               END IF
            END ASSOCIATE
# 137 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
            CALL timestop(handle)
         END SUBROUTINE pw_create_c3d_gs
# 140 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"
# 141 "/builddir/build/BUILD/cp2k-2025.1-build/cp2k-2025.1/src/pw/pw_types.F"

END MODULE pw_types
